// This is a CUDA program that does the following:
//
// 1. On the host, fill the indices array with the index of each rectangle
// 2. Copy the indices array from the host to the device
// 3. Use the device to run a kernel that:
//    a. Uses the indices array to determine the left x-coordinate of
//       each rectangle (float const x = deviceIndices[threadId] * width)
//    b. Uses the left x-coordinate to determine the height of the rectangle
//       (float const height = sqrtf(1.0 - x * x)
//    b. Uses the width and height to determine the area
//       (deviceAreas[threadId] = width * height)
// 4. Copy the areas array from the device to the host
// 5. On the host, sum the areas and multiply by 4 to calculate an approximation
//    of pi
// 6. On the host, print the resulting approximation of pi
//
// Author: Aaron Weeden, Shodor, 2016

// Import library so we can call printf()
#include <stdio.h>

// Import library so we can call exit(), malloc(), free(), etc.
#include <stdlib.h>

// Define the number of rectangles
#define RECT_COUNT 10

// Define the number of bytes in each array (depending on the type)
#define INT_BYTE_COUNT   ((RECT_COUNT) * sizeof(int))
#define FLOAT_BYTE_COUNT ((RECT_COUNT) * sizeof(float))

// Declare functions that will be defined later
void TryMalloc(void * const err);

// Start the program
int main()
{
  // Define a variable for the width of a rectangle
  float width = (float)1.0 / RECT_COUNT;

  // Declare variables for the host and device arrays
  int   * hostIndices;
  float * hostAreas;
  int   * deviceIndices;
  float * deviceAreas;

  // Allocate memory for the host arrays

  // Allocate memory for the device arrays

  // On the host, fill the indices array with the index of each rectangle
  for (int i = 0; i < RECT_COUNT; i++)
  {
    hostIndices[i] = i;
  }

  // Copy the indices array from the host to the device

  // Use the device to run a kernel

  // Copy the areas array from the device to the host

  // On the host, sum the areas and multiply by 4 to calculate an approximation
  // of pi
  float pi = 0.0;
  for (int i = 0; i < RECT_COUNT; i++)
  {
    pi += hostAreas[i];
  }
  pi *= 4.0;

  // On the host, print the resulting approximation of pi
  printf("Result:\tpi = %f\n", pi);

  // De-allocate memory for the device arrays

  // De-allocate memory for the host arrays

  return 0;
}

// Define a function to check whether a malloc() call was successful
void TryMalloc(void * const err)
{
  if (err == NULL)
  {
    fprintf(stderr, "malloc error\n");
    exit(EXIT_FAILURE);
  }
}

